@@ -5,17 +5,20 @@ from .svdpp import SVDpp
 from .als import ALS
 from .bpr import BPR
 from .ncf import NCF
-from .youtube_retrieval import YouTuBeRetrieval
+from .youtube_retrieval import YouTubeRetrieval
 from .youtube_ranking import YouTubeRanking
 from .fm import FM
 from .wide_deep import WideDeep
 from .deepfm import DeepFM
 from .autoint import AutoInt
 from .din import DIN
-from .knn_embed import KnnEmbedding, KnnEmbeddingApproximate
 from .rnn4rec import RNN4Rec
 from .caser import Caser
 from .wave_net import WaveNet
+from .item2vec import Item2Vec
+from .deepwalk import DeepWalk
+from .ngcf import NGCF
+from .lightgcn import LightGCN
 
 __all__ = [
     "UserCF",
@@ -25,16 +28,18 @@ __all__ = [
     "ALS",
     "BPR",
     "NCF",
-    "YouTuBeRetrieval",
+    "YouTubeRetrieval",
     "YouTubeRanking",
     "FM",
     "WideDeep",
     "DeepFM",
     "AutoInt",
     "DIN",
-    "KnnEmbedding",
-    "KnnEmbeddingApproximate",
     "RNN4Rec",
     "Caser",
-    "WaveNet"
+    "WaveNet",
+    "Item2Vec",
+    "DeepWalk",
+    "NGCF",
+    "LightGCN",
 ]
