@@ -1,9 +1,9 @@
 from collections import defaultdict, OrderedDict
+
 import numpy as np
 
 
-def col_name2index(user_col=None, item_col=None,
-                   sparse_col=None, dense_col=None):
+def col_name2index(user_col=None, item_col=None, sparse_col=None, dense_col=None):
     # format: {column_family_name: {column_name: index}}
     # if no such family, default format would be: {column_family_name: {[]: []}
     name_mapping = defaultdict(OrderedDict)
@@ -23,9 +23,7 @@ def col_name2index(user_col=None, item_col=None,
     if user_col and dense_col:
         user_dense_col = _extract_common_col(dense_col, user_col)
         for col in user_dense_col:
-            name_mapping["user_dense_col"].update(
-                {col: name_mapping["dense_col"][col]}
-            )
+            name_mapping["user_dense_col"].update({col: name_mapping["dense_col"][col]})
 
     if item_col and sparse_col:
         item_sparse_col = _extract_common_col(sparse_col, item_col)
@@ -36,9 +34,7 @@ def col_name2index(user_col=None, item_col=None,
     if item_col and dense_col:
         item_dense_col = _extract_common_col(dense_col, item_col)
         for col in item_dense_col:
-            name_mapping["item_dense_col"].update(
-                {col: name_mapping["dense_col"][col]}
-            )
+            name_mapping["item_dense_col"].update({col: name_mapping["dense_col"][col]})
 
     return name_mapping
 
@@ -47,7 +43,7 @@ def _extract_common_col(col1, col2):
     # np.intersect1d will return the sorted common column names,
     # but we also want to preserve the original order of common column in
     # col1 and col2
-    common_col, indices_in_col1, _ = np.intersect1d(col1, col2,
-                                                    assume_unique=True,
-                                                    return_indices=True)
+    common_col, indices_in_col1, _ = np.intersect1d(
+        col1, col2, assume_unique=True, return_indices=True
+    )
     return common_col[np.lexsort((common_col, indices_in_col1))]
