@@ -1,7 +1,9 @@
 from math import floor
 from random import random, seed as set_random_seed
+
 import numpy as np
 from tqdm import tqdm
+
 from ..utils.misc import time_block
 
 
@@ -21,8 +23,7 @@ class SamplingBase(object):
         }
         # sample negative items for every user
         with time_block("random neg item sampling"):
-            for u, i in zip(self.dataset.user_indices,
-                            self.dataset.item_indices):
+            for u, i in zip(self.dataset.user_indices, self.dataset.item_indices):
                 item_indices_sampled.append(i)
                 for _ in range(self.num_neg):
                     item_neg = floor(n_items * random())
@@ -55,7 +56,8 @@ class SamplingBase(object):
                 neg_size = len(u_consumed) * self.num_neg
 
                 neg_sampled = np.random.choice(
-                    items, size=neg_size, p=item_prob, replace=True)
+                    items, size=neg_size, p=item_prob, replace=True
+                )
                 item_indices_sampled.extend(neg_sampled)
 
         item_indices_sampled = np.asarray(item_indices_sampled)
@@ -72,17 +74,15 @@ class SamplingBase(object):
 
 
 class NegativeSampling(SamplingBase):
-    def __init__(self, dataset, data_info, num_neg, sparse=None, dense=None,
-                 batch_sampling=False):
+    def __init__(
+        self, dataset, data_info, num_neg, sparse=None, dense=None, batch_sampling=False
+    ):
         super(NegativeSampling, self).__init__(dataset, data_info, num_neg)
-
         if batch_sampling and dataset.has_sampled:
             self.user_indices = dataset.user_indices_orig
             self.item_indices = dataset.item_indices_orig
-            self.sparse_indices = (
-                dataset.sparse_indices_orig if sparse else None)
-            self.dense_values = (
-                dataset.dense_values_orig if dense else None)
+            self.sparse_indices = dataset.sparse_indices_orig if sparse else None
+            self.dense_values = dataset.dense_values_orig if dense else None
         else:
             self.user_indices = dataset.user_indices
             self.item_indices = dataset.item_indices
@@ -93,10 +93,7 @@ class NegativeSampling(SamplingBase):
         self.dense = dense
 
     def generate_all(self, seed=42, item_gen_mode="random"):
-        user_indices_sampled = np.repeat(
-            self.user_indices, self.num_neg + 1, axis=0
-        )
-
+        user_indices_sampled = np.repeat(self.user_indices, self.num_neg + 1, axis=0)
         if item_gen_mode not in ["random", "popular"]:
             raise ValueError(
                 "sampling item_gen_mode must either be 'random' or 'popular'"
@@ -106,12 +103,16 @@ class NegativeSampling(SamplingBase):
         elif item_gen_mode == "popular":
             item_indices_sampled = self.sample_items_popular(seed=seed)
 
-        sparse_indices_sampled = self._sparse_indices_sampling(
-            self.sparse_indices, item_indices_sampled
-        ) if self.sparse else None
-        dense_values_sampled = self._dense_values_sampling(
-            self.dense_values, item_indices_sampled
-        ) if self.dense else None
+        sparse_indices_sampled = (
+            self._sparse_indices_sampling(self.sparse_indices, item_indices_sampled)
+            if self.sparse
+            else None
+        )
+        dense_values_sampled = (
+            self._dense_values_sampling(self.dense_values, item_indices_sampled)
+            if self.dense
+            else None
+        )
         label_sampled = self._label_negative_sampling(self.data_size)
 
         return (
@@ -119,16 +120,14 @@ class NegativeSampling(SamplingBase):
             item_indices_sampled,
             label_sampled,
             sparse_indices_sampled,
-            dense_values_sampled
+            dense_values_sampled,
         )
 
     def __call__(self, shuffle=True, batch_size=None):
         if shuffle:
             mask = np.random.permutation(range(self.data_size))
-            self.sparse_indices = (
-                self.sparse_indices[mask] if self.sparse else None)
-            self.dense_values = (
-                self.dense_values[mask] if self.dense else None)
+            self.sparse_indices = self.sparse_indices[mask] if self.sparse else None
+            self.dense_values = self.dense_values[mask] if self.dense else None
 
         user_consumed = {
             u: set(items) for u, items in self.data_info.user_consumed.items()
@@ -137,20 +136,20 @@ class NegativeSampling(SamplingBase):
         return self.sample_batch(user_consumed, n_items, batch_size)
 
     def sample_batch(self, user_consumed, n_items, batch_size):
-        for k in tqdm(range(0, self.data_size, batch_size),
-                      desc="batch_sampling train"):
+        for k in tqdm(
+            range(0, self.data_size, batch_size), desc="batch_sampling train"
+        ):
             batch_slice = slice(k, k + batch_size)
             batch_user_indices = self.user_indices[batch_slice]
             batch_item_indices = self.item_indices[batch_slice]
             batch_sparse_indices = (
-                self.sparse_indices[batch_slice] if self.sparse else None)
-            batch_dense_values = (
-                self.dense_values[batch_slice] if self.dense else None)
+                self.sparse_indices[batch_slice] if self.sparse else None
+            )
+            batch_dense_values = self.dense_values[batch_slice] if self.dense else None
 
             user_indices_sampled = np.repeat(
                 batch_user_indices, self.num_neg + 1, axis=0
             )
-
             item_indices_sampled = list()
             for u, i in zip(batch_user_indices, batch_item_indices):
                 item_indices_sampled.append(i)
@@ -161,22 +160,26 @@ class NegativeSampling(SamplingBase):
                     item_indices_sampled.append(item_neg)
             item_indices_sampled = np.array(item_indices_sampled)
 
-            sparse_indices_sampled = self._sparse_indices_sampling(
-                batch_sparse_indices, item_indices_sampled
-            ) if self.sparse else None
-            dense_values_sampled = self._dense_values_sampling(
-                batch_dense_values, item_indices_sampled
-            ) if self.dense else None
-            label_sampled = self._label_negative_sampling(
-                len(batch_user_indices)
+            sparse_indices_sampled = (
+                self._sparse_indices_sampling(
+                    batch_sparse_indices, item_indices_sampled
+                )
+                if self.sparse
+                else None
             )
+            dense_values_sampled = (
+                self._dense_values_sampling(batch_dense_values, item_indices_sampled)
+                if self.dense
+                else None
+            )
+            label_sampled = self._label_negative_sampling(len(batch_user_indices))
 
             yield (
                 user_indices_sampled,
                 item_indices_sampled,
                 label_sampled,
                 sparse_indices_sampled,
-                dense_values_sampled
+                dense_values_sampled,
             )
 
     def _sparse_indices_sampling(self, sparse_indices, item_indices_sampled):
@@ -184,40 +187,43 @@ class NegativeSampling(SamplingBase):
         item_sparse_col = self.data_info.item_sparse_col.index
 
         if user_sparse_col and item_sparse_col:
-            user_sparse_indices = np.take(
-                sparse_indices, user_sparse_col, axis=1)
+            user_sparse_indices = np.take(sparse_indices, user_sparse_col, axis=1)
             user_sparse_sampled = np.repeat(
-                user_sparse_indices, self.num_neg + 1, axis=0)
+                user_sparse_indices, self.num_neg + 1, axis=0
+            )
             item_sparse_sampled = self.data_info.item_sparse_unique[
-                item_indices_sampled]
+                item_indices_sampled
+            ]
 
-            assert len(user_sparse_sampled) == len(item_sparse_sampled), (
-                "num of user sampled must equal to num of item sampled")
+            assert len(user_sparse_sampled) == len(
+                item_sparse_sampled
+            ), "num of user sampled must equal to num of item sampled"
             # keep column names in original order
             orig_cols = user_sparse_col + item_sparse_col
             col_reindex = np.arange(len(orig_cols))[np.argsort(orig_cols)]
-            return np.concatenate(
-                [user_sparse_sampled, item_sparse_sampled], axis=-1
-            )[:, col_reindex]
+            return np.concatenate([user_sparse_sampled, item_sparse_sampled], axis=-1)[
+                :, col_reindex
+            ]
 
         elif user_sparse_col:
-            user_sparse_indices = np.take(
-                sparse_indices, user_sparse_col, axis=1)
+            user_sparse_indices = np.take(sparse_indices, user_sparse_col, axis=1)
             user_sparse_sampled = np.repeat(
-                user_sparse_indices, self.num_neg + 1, axis=0)
+                user_sparse_indices, self.num_neg + 1, axis=0
+            )
             return user_sparse_sampled
 
         elif item_sparse_col:
             item_sparse_sampled = self.data_info.item_sparse_unique[
-                item_indices_sampled]
+                item_indices_sampled
+            ]
             return item_sparse_sampled
 
-    def _dense_indices_sampling(self, item_indices_sampled):
-        n_samples = len(item_indices_sampled)
-        user_dense_col = self.data_info.user_dense_col.index
-        item_dense_col = self.data_info.item_dense_col.index
-        total_dense_cols = len(user_dense_col) + len(item_dense_col)
-        return np.tile(np.arange(total_dense_cols), [n_samples, 1])
+    # def _dense_indices_sampling(self, item_indices_sampled):
+    #    n_samples = len(item_indices_sampled)
+    #    user_dense_col = self.data_info.user_dense_col.index
+    #    item_dense_col = self.data_info.item_dense_col.index
+    #    total_dense_cols = len(user_dense_col) + len(item_dense_col)
+    #    return np.tile(np.arange(total_dense_cols), [n_samples, 1])
 
     def _dense_values_sampling(self, dense_values, item_indices_sampled):
         user_dense_col = self.data_info.user_dense_col.index
@@ -225,36 +231,31 @@ class NegativeSampling(SamplingBase):
 
         if user_dense_col and item_dense_col:
             user_dense_values = np.take(dense_values, user_dense_col, axis=1)
-            user_dense_sampled = np.repeat(
-                user_dense_values, self.num_neg + 1, axis=0)
-            item_dense_sampled = self.data_info.item_dense_unique[
-                item_indices_sampled]
-
-            assert len(user_dense_sampled) == len(item_dense_sampled), (
-                "num of user sampled must equal to num of item sampled")
+            user_dense_sampled = np.repeat(user_dense_values, self.num_neg + 1, axis=0)
+            item_dense_sampled = self.data_info.item_dense_unique[item_indices_sampled]
+            assert len(user_dense_sampled) == len(
+                item_dense_sampled
+            ), "num of user sampled must equal to num of item sampled"
             # keep column names in original order
             orig_cols = user_dense_col + item_dense_col
             col_reindex = np.arange(len(orig_cols))[np.argsort(orig_cols)]
-            return np.concatenate(
-                [user_dense_sampled, item_dense_sampled], axis=-1
-            )[:, col_reindex]
+            return np.concatenate([user_dense_sampled, item_dense_sampled], axis=-1)[
+                :, col_reindex
+            ]
 
         elif user_dense_col:
             user_dense_values = np.take(dense_values, user_dense_col, axis=1)
-            user_dense_sampled = np.repeat(
-                user_dense_values, self.num_neg + 1, axis=0)
+            user_dense_sampled = np.repeat(user_dense_values, self.num_neg + 1, axis=0)
             return user_dense_sampled
 
         elif item_dense_col:
-            item_dense_sampled = self.data_info.item_dense_unique[
-                item_indices_sampled]
+            item_dense_sampled = self.data_info.item_dense_unique[item_indices_sampled]
             return item_dense_sampled
 
 
 class PairwiseSampling(SamplingBase):
     def __init__(self, dataset, data_info, num_neg=1):
         super(PairwiseSampling, self).__init__(dataset, data_info, num_neg)
-
         if dataset.has_sampled:
             self.user_indices = dataset.user_indices_orig
             self.item_indices = dataset.item_indices_orig
@@ -276,8 +277,7 @@ class PairwiseSampling(SamplingBase):
         return self.sample_batch(user_consumed_set, n_items, batch_size)
 
     def sample_batch(self, user_consumed_set, n_items, batch_size):
-        for k in tqdm(range(0, self.data_size, batch_size),
-                      desc="pair_sampling train"):
+        for k in tqdm(range(0, self.data_size, batch_size), desc="pair_sampling train"):
             batch_slice = slice(k, k + batch_size)
             batch_user_indices = self.user_indices[batch_slice]
             batch_item_indices_pos = self.item_indices[batch_slice]
@@ -290,17 +290,12 @@ class PairwiseSampling(SamplingBase):
                 batch_item_indices_neg.append(item_neg)
 
             batch_item_indices_neg = np.asarray(batch_item_indices_neg)
-            yield (
-                batch_user_indices,
-                batch_item_indices_pos,
-                batch_item_indices_neg
-            )
+            yield (batch_user_indices, batch_item_indices_pos, batch_item_indices_neg)
 
 
 class PairwiseSamplingSeq(PairwiseSampling):
     def __init__(self, dataset, data_info, num_neg=1, mode=None, num=None):
         super(PairwiseSamplingSeq, self).__init__(dataset, data_info, num_neg)
-
         self.seq_mode = mode
         self.seq_num = num
         self.n_items = data_info.n_items
@@ -310,23 +305,21 @@ class PairwiseSamplingSeq(PairwiseSampling):
         # avoid circular import
         from ..data.sequence import user_interacted_seq
 
-        for k in tqdm(range(0, self.data_size, batch_size),
-                      desc="pair_sampling sequence train"):
+        for k in tqdm(
+            range(0, self.data_size, batch_size), desc="pair_sampling sequence train"
+        ):
             batch_slice = slice(k, k + batch_size)
             batch_user_indices = self.user_indices[batch_slice]
             batch_item_indices_pos = self.item_indices[batch_slice]
 
-            (
-                batch_interacted,
-                batch_interacted_len
-            ) = user_interacted_seq(
+            (batch_interacted, batch_interacted_len) = user_interacted_seq(
                 batch_user_indices,
                 batch_item_indices_pos,
                 self.user_consumed,
                 self.n_items,
                 self.seq_mode,
                 self.seq_num,
-                user_consumed_set
+                user_consumed_set,
             )
 
             batch_item_indices_neg = list()
@@ -342,5 +335,5 @@ class PairwiseSamplingSeq(PairwiseSampling):
                 batch_item_indices_pos,
                 batch_item_indices_neg,
                 batch_interacted,
-                batch_interacted_len
+                batch_interacted_len,
             )
